from test_all_fixers import lib3to2FixerTestCase

class Test_str(lib3to2FixerTestCase):
    fixer = u"str"

    def test_str_call(self):
        b = u"""str(x, y, z)"""
        a = u"""unicode(x, y, z)"""
        self.check(b, a)

    def test_chr_call(self):
        b = u"""chr(a, t, m)"""
        a = u"""unichr(a, t, m)"""
        self.check(b, a)

    def test_str_literal_1(self):
        b = u'''"x"'''
        a = u'''u"x"'''
        self.check(b, a)

    def test_str_literal_2(self):
        b = u"""r'x'"""
        a = u"""ur'x'"""
        self.check(b, a)

    def test_str_literal_3(self):
        b = u"""R'''x'''"""
        a = u"""uR'''x'''"""
        self.check(b, a)




from test_all_fixers import lib3to2FixerTestCase

class Test_imports(lib3to2FixerTestCase):
    fixer = u"imports"

    def test_various_unchanged(self):

        # Enclosed in a string
        s = u"'import queue'"
        self.unchanged(s)

        # Never was imported
        s = u"print(queue)"
        self.unchanged(s)

    def test_all_nodotted_names_solo(self):

        b = u"import configparser"
        a = u"import ConfigParser"
        self.check(b, a)

        b = u"from winreg import *"
        a = u"from _winreg import *"
        self.check(b, a)

        b = u"import copyreg"
        a = u"import copy_reg"
        self.check(b, a)

        b = u"import queue"
        a = u"import Queue"
        self.check(b, a)

        b = u"import socketserver"
        a = u"import SocketServer"
        self.check(b, a)

        b = u"import _markupbase"
        a = u"import markupbase"
        self.check(b, a)

    def test_nodotted_names_duo(self):

        b = u"import configparser, copyreg"
        a = u"import ConfigParser, copy_reg"
        self.check(b, a)

        b = u"import _markupbase, queue as bob"
        a = u"import markupbase, Queue as bob"
        self.check(b, a)

    def test_nodotted_names_quad(self):

        b = u"import configparser, winreg, socketserver, _markupbase"
        a = u"import ConfigParser, _winreg, SocketServer, markupbase"
        self.check(b, a)

        b = u"import queue, math, _markupbase, copyreg"
        a = u"import Queue, math, markupbase, copy_reg"
        self.check(b, a)

    def test_all_dotted_names_solo(self):

        b = u"import dbm.bsd as bsd"
        a = u"import dbhash as bsd"
        self.check(b, a)

        b = u"import dbm.ndbm"
        a = u"import dbm"
        self.check(b, a)

        b = u"import dbm.dumb"
        a = u"import dumbdbm"
        self.check(b, a)

        b = u"from dbm import gnu"
        a = u"import gdbm as gnu"
        self.check(b, a)

        b = u"import html.parser"
        a = u"import HTMLParser"
        self.check(b, a)

        b = u"import html.entities"
        a = u"import htmlentitydefs"
        self.check(b, a)

        b = u"from http import client"
        a = u"import httplib as client"
        self.check(b, a)

        b = u"import http.cookies"
        a = u"import Cookie"
        self.check(b, a)

        b = u"import http.cookiejar"
        a = u"import cookielib"
        self.check(b, a)

        b = u"import tkinter.dialog"
        a = u"import Dialog"
        self.check(b, a)

        b = u"import tkinter._fix"
        a = u"import FixTk"
        self.check(b, a)

        b = u"import tkinter.scrolledtext"
        a = u"import ScrolledText"
        self.check(b, a)

        b = u"import tkinter.tix"
        a = u"import Tix"
        self.check(b, a)

        b = u"import tkinter.constants"
        a = u"import Tkconstants"
        self.check(b, a)

        b = u"import tkinter.dnd"
        a = u"import Tkdnd"
        self.check(b, a)

        b = u"import tkinter.__init__"
        a = u"import Tkinter"
        self.check(b, a)

        #TODO: Make this work (see the fix_imports)
        #b = "import tkinter"
        #a = "import Tkinter"
        #self.check(b, a)

        b = u"import tkinter.colorchooser"
        a = u"import tkColorChooser"
        self.check(b, a)

        b = u"import tkinter.commondialog"
        a = u"import tkCommonDialog"
        self.check(b, a)

        b = u"from tkinter.font import *"
        a = u"from tkFont import *"
        self.check(b, a)

        b = u"import tkinter.messagebox"
        a = u"import tkMessageBox"
        self.check(b, a)

        b = u"import tkinter.turtle"
        a = u"import turtle"
        self.check(b, a)

        b = u"import urllib.robotparser"
        a = u"import robotparser"
        self.check(b, a)

        b = u"import test.support"
        a = u"import test.test_support"
        self.check(b, a)

        b = u"from test import support"
        a = u"from test import test_support as support"
        self.check(b, a)

        b = u"import xmlrpc.client"
        a = u"import xmlrpclib"
        self.check(b, a)

    def test_dotted_names_duo(self):

        b = u"import   tkinter.font,  dbm.bsd"
        a = u"import   tkFont,  dbhash"
        self.check(b, a)

        b = u"import test.support,  http.cookies"
        a = u"import test.test_support,  Cookie"
        self.check(b, a)

    def test_dotted_names_quad(self):

        b = u"import    html.parser as spam,  math,     tkinter.__init__,   dbm.gnu #comment!"
        a = u"import    HTMLParser as spam,  math,     Tkinter,   gdbm #comment!"
        self.check(b, a)

        b = u"import math, tkinter.dnd, dbm.ndbm as one, dbm.ndbm as two, urllib"
        a = u"import math, Tkdnd, dbm as one, dbm as two, urllib"
        self.check(b, a)

    def test_usage(self):

        b = u"""
        import queue as james
        james.do_stuff()"""
        a = u"""
        import Queue as james
        james.do_stuff()"""
        self.check(b, a)

        b = u"""
        import queue
        queue.do_stuff()"""
        a = u"""
        import Queue
        Queue.do_stuff()"""
        self.check(b, a)

        b = u"""
        import dbm.gnu
        dbm.gnu.open('generic_file')"""
        a = u"""
        import gdbm
        gdbm.open('generic_file')"""
        self.check(b, a)

        b = u"""
        import tkinter.dialog, tkinter.colorchooser
        tkinter = tkinter.dialog(tkinter.colorchooser("Just messing around"))
        tkinter.test_should_work = True
        tkinter.dialog.dont.code.like.this = True"""
        a = u"""
        import Dialog, tkColorChooser
        tkinter = Dialog(tkColorChooser("Just messing around"))
        tkinter.test_should_work = True
        Dialog.dont.code.like.this = True"""
        self.check(b, a)



from test_all_fixers import lib3to2FixerTestCase

class Test_funcattrs(lib3to2FixerTestCase):
    fixer = u"funcattrs"

    def test_doc_unchanged(self):
        b = u"""whats.up.__doc__"""
        self.unchanged(b)
    def test_defaults(self):
        b = u"""myFunc.__defaults__"""
        a = u"""myFunc.func_defaults"""
        self.check(b, a)
    def test_closure(self):
        b = u"""fore.__closure__"""
        a = u"""fore.func_closure"""
        self.check(b, a)
    def test_globals(self):
        b = u"""funkFunc.__globals__"""
        a = u"""funkFunc.func_globals"""
        self.check(b, a)
    def test_dict_unchanged(self):
        b = u"""tricky.__dict__"""
        self.unchanged(b)
    def test_name_unchanged(self):
        b = u"""sayMy.__name__"""
        self.unchanged(b)



from test_all_fixers import lib3to2FixerTestCase

class Test_methodattrs(lib3to2FixerTestCase):
    fixer = u"methodattrs"

    attrs = [u"func", u"self"]

    def test_methodattrs(self):
        for attr in self.attrs:
            b = u"a.__%s__" % attr
            a = u"a.im_%s" % attr
            self.check(b, a)

            b = u"self.foo.__%s__.foo_bar" % attr
            a = u"self.foo.im_%s.foo_bar" % attr
            self.check(b, a)

        b = u"dir(self.foo.__self__.__class__)"
        a = u"dir(self.foo.im_self.__class__)"
        self.check(b, a)

    def test_unchanged(self):
        for attr in self.attrs:
            s = u"foo(__%s__ + 5)" % attr
            self.unchanged(s)



from test_all_fixers import lib3to2FixerTestCase

class Test_numliterals(lib3to2FixerTestCase):
    fixer = u"numliterals"

    def test_octal_1(self):
        b = u"""0o755"""
        a = u"""0755"""
        self.check(b, a)

    def test_octal_2(self):
        b = u"""0o777"""
        a = u"""0777"""
        self.check(b, a)

    def test_bin_1(self):
        b = u"""0b10010110"""
        a = u"""__builtins__.long("10010110", 2)"""
        self.check(b, a)

    def test_bin_2(self):
        b = u"""spam(0b1101011010110)"""
        a = u"""spam(__builtins__.long("1101011010110", 2))"""
        self.check(b, a)

    def test_comments_and_spacing_2(self):
        b = u"""b = 0o755 # spam"""
        a = u"""b = 0755 # spam"""
        self.check(b, a)

    def test_unchanged_str(self):
        s = u"""'0x1400'"""
        self.unchanged(s)

        s = u"""'0b011000'"""
        self.unchanged(s)

        s = u"""'0o755'"""
        self.unchanged(s)

    def test_unchanged_other(self):
        s = u"""5.0"""
        self.unchanged(s)

        s = u"""5.0e10"""
        self.unchanged(s)

        s = u"""5.4 + 4.9j"""
        self.unchanged(s)

        s = u"""4j"""
        self.unchanged(s)

        s = u"""4.4j"""
        self.unchanged(s)



from lib2to3.pygram import token, python_symbols as syms
from lib2to3.pytree import Leaf, Node
from lib2to3.fixer_util import Comma, Name

def Star(prefix=None):
    return Leaf(token.STAR, u'*', prefix=prefix)

def commatize(leafs):
    u"""
    Accepts/turns: (Name, Name, ..., Name, Name) 
    Returns/into: (Name, Comma, Name, Comma, ..., Name, Comma, Name)
    """
    new_leafs = []
    for leaf in leafs:
        new_leafs.append(leaf)
        new_leafs.append(Comma())
    del new_leafs[-1]
    return new_leafs

def indentation(node):
    u"""
    Recursively checks every previous sibling for token.INDENT

    Returns empty string if not found, or the indentation if it is.
    """
    prev_sibling = node.prev_sibling
    if node.type == token.INDENT:
        return node.value
    else:
        return indentation(prev_sibling) if prev_sibling is not None else u""

def NameImport(package, as_name=None, prefix=None):
    u"""
    Accepts a package (Name node), name to import it as (string), and
    optional prefix and returns a node:
    import <package> [as <as_name>]
    """
    if prefix is None:
        prefix = u""
    children = [Name(u"import", prefix=prefix), package]
    if as_name is not None:
        children.extend([Name(u"as", prefix=u" "),
                         Name(as_name, prefix=u" ")])
    return Node(syms.import_name, children)

_compound_stmts = (syms.if_stmt, syms.while_stmt, syms.for_stmt, syms.try_stmt, syms.with_stmt)
_import_stmts = (syms.import_name, syms.import_from)
def import_binding_scope(node):
    u"""
    Generator yields all nodes for which a node (an import_stmt) has scope
    The purpose of this is for a call to _find() on each of them
    """
    # import_name / import_from are small_stmts
    assert node.type in _import_stmts
    test = node.next_sibling
    # A small_stmt can only be followed by a SEMI or a NEWLINE.
    while test.type == token.SEMI:
        nxt = test.next_sibling
        # A SEMI can only be followed by a small_stmt or a NEWLINE
        if nxt.type == token.NEWLINE:
            break
        else:
            yield nxt
        # A small_stmt can only be followed by either a SEMI or a NEWLINE
        test = nxt.next_sibling
    # Covered all subsequent small_stmts after the import_stmt
    # Now to cover all subsequent stmts after the parent simple_stmt
    parent = node.parent
    assert parent.type == syms.simple_stmt
    test = parent.next_sibling
    while test is not None:
        # Yes, this will yield NEWLINE and DEDENT.  Deal with it.
        yield test
        test = test.next_sibling

    context = parent.parent
    # Recursively yield nodes following imports inside of a if/while/for/try/with statement
    if context.type in _compound_stmts:
        # import is in a one-liner
        c = context
        while c.next_sibling is not None:
            yield c.next_sibling
            c = c.next_sibling
        context = context.parent

    # Can't chain one-liners on one line, so that takes care of that.

    p = context.parent
    if p is None:
        return

    # in a multi-line suite

    while p.type in _compound_stmts:

        if context.type == syms.suite:
            yield context

        context = context.next_sibling

        if context is None:
            context = p.parent
            p = context.parent
            if p is None:
                break

def ImportAsName(name, as_name, prefix=None):
    new_name = Name(name)
    new_as = Name(u"as", prefix=u" ")
    new_as_name = Name(as_name, prefix=u" ")
    new_node = Node(syms.import_as_name, [new_name, new_as, new_as_name])
    if prefix is not None:
        new_node.prefix = prefix
    return new_node



u"""
Fixer for print: from __future__ import print_function.
"""

from lib2to3 import fixer_base
from lib2to3.pytree import Node
from lib2to3.pygram import python_symbols as syms
from lib2to3.fixer_util import Name, FromImport, Newline

class FixPrintfunction(fixer_base.BaseFix):

    explicit = True # Not the preferred way to fix print

    def start_tree(self, tree, filename):
        u"""This is only run once; we want to remember the first node"""
        super(FixPrintfunction, self).start_tree(tree, filename)
        self._tree = tree
        self.have_print = False

    PATTERN = u"""
              power< 'print' trailer < '(' any* ')' > any* >
              """

    def match(self, node):
        u"""
        Since the tree needs to be fixed once and only once if and only if it
        matches, then we can start discarding matches after we make the first.
        """
        return not self.have_print and super(FixPrintfunction, self).match(node)

    def transform(self, node, results):
        tree = self._tree
        self.have_print = True
        future_stmt = FromImport(u"__future__",
                                [Name(u"print_function", prefix=u" ")])
        children = tree.children[:]
        new_node = Node(syms.simple_stmt, [future_stmt, Newline()])
        for child in children:
            child.remove()
            new_node.append_child(child)
        tree.insert_child(0, new_node)



u"""
Fixer to remove class decorators
"""

from lib2to3 import fixer_base
from lib2to3.fixer_util import Call, Assign

def dec_name(decorator_node):
    return decorator_node.children[1]

def get_pos(node):
    assert node.parent, repr(node)
    for i, child in enumerate(node.parent.children):
        if child is node:
            return i

def call_chain(inner_name, names):
    u"""
    Accepts an inner name and a sequence
    Returns a chain of calls by that sequence on the inner name.
    e.g., accepts d, [a,b,c] and returns a(b(c(d)))
    """
    if len(names) > 0:
        name = names[0]
        rest = names[1:]
        return Call(name.clone(), [call_chain(inner_name, rest)])
    else:
        return inner_name.clone()
            
class FixClassdecorator(fixer_base.BaseFix):

    PATTERN = u"""
              dec_node=decorated < dec=decorator < any* > cls=classdef < 'class' name=any any* > > | 
              dec_node=decorated < decs=decorators < any* > cls=classdef < 'class' name=any any* > >
              """

    def transform(self, node, results):
        u"""
        This removes the decorators and replaces them
        with the syntactical equivalent (source: PEP 3129)        
        """
        dec_node = results[u"dec_node"]
        cls = results[u"cls"]
        name = results[u"name"].clone()
        name.prefix = u""
        # If there's just one decorator, put it in a list
        # Otherwise, get all the decorators into one list
        decs = [results.get(u"dec")] if u"dec" in results else \
               [dec for dec in results.get(u"decs").children]
        # where to insert the new name after decorations are removed
        pos = get_pos(cls)+1
        dec_names = [dec_name(dec).clone() for dec in decs]
        for dec in decs:
            dec.remove()
        # new_name becomes first_decorator(second_decorator(...(class_name)...))
        new_name = call_chain(name, dec_names)
        new_assignment = Assign(name, new_name)
        cls.parent.insert_child(pos, new_assignment)



u"""
Fixer for bytes -> str.
"""

import re
from lib2to3.pgen2 import token
from lib2to3 import fixer_base
from lib2to3.fixer_util import Name

_literal_re = re.compile(ur"[bB][rR]?[\'\"]")

class FixBytes(fixer_base.BaseFix):

    PATTERN = u"STRING | 'bytes'"

    def transform(self, node, results):
        new = node.clone()
        if node.type == token.NAME:
            assert new.value == u"bytes"
            new.value = u"str"
            return new
        elif node.type == token.STRING:
            if _literal_re.match(node.value):
                new.value = new.value[1:]
                return new



from test_all_fixers import lib3to2FixerTestCase

class Test_bytes(lib3to2FixerTestCase):
    fixer = u"bytes"

    def test_bytes_call_1(self):
        b = u"""bytes(x, y, z)"""
        a = u"""str(x, y, z)"""
        self.check(b, a)

    def test_bytes_call_2(self):
        b = u"""bytes(x, y, z) + b"florist" """
        a = u"""str(x, y, z) + "florist" """
        self.check(b, a)

    def test_bytes_literal_1(self):
        b = u'''b"\x41"'''
        a = u'''"\x41"'''
        self.check(b, a)

    def test_bytes_literal_2(self):
        b = u"""b'x'"""
        a = u"""'x'"""
        self.check(b, a)

    def test_bytes_literal_3(self):
        b = u"""BR'''\x13'''"""
        a = u"""R'''\x13'''"""
        self.check(b, a)

    def test_bytes_concatenation(self):
        b = u"""b'bytes' + b'bytes'"""
        a = u"""'bytes' + 'bytes'"""
        self.check(b, a)
